@@ -37,10 +37,6 @@ typedef struct _GarconMenuMergerContext GarconMenuMergerContext;
 
 
 
-#define GARCON_MENU_MERGER_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GARCON_TYPE_MENU_MERGER, GarconMenuMergerPrivate))
-
-
-
 /* Property identifiers */
 enum
 {
@@ -145,7 +141,7 @@ garcon_menu_merger_provider_init (GarconMenuTreeProviderIface *iface)
 static void
 garcon_menu_merger_init (GarconMenuMerger *merger)
 {
-  merger->priv = GARCON_MENU_MERGER_GET_PRIVATE (merger);
+  merger->priv = G_TYPE_INSTANCE_GET_PRIVATE (merger, GARCON_TYPE_MENU_MERGER, GarconMenuMergerPrivate);
   merger->priv->tree_provider = NULL;
   merger->priv->menu = NULL;
   merger->priv->file_stack = NULL;
@@ -335,8 +331,7 @@ garcon_menu_merger_run (GarconMenuMerger *merger,
   garcon_menu_merger_clean_up_elements (merger->priv->menu, GARCON_MENU_NODE_TYPE_DEFAULT_LAYOUT);
   garcon_menu_merger_clean_up_elements (merger->priv->menu, GARCON_MENU_NODE_TYPE_LAYOUT);
 
-  g_list_foreach (context.file_stack, (GFunc) g_object_unref, NULL);
-  g_list_free (context.file_stack);
+  _garcon_g_list_free_full (context.file_stack, g_object_unref);
 
   return context.success;
 }
@@ -660,9 +655,9 @@ static void
 garcon_menu_merger_remove_duplicate_paths (GNode             *node,
                                            GarconMenuNodeType type)
 {
-  GList *destroy_nodes = NULL;
-  GList *remaining_nodes = NULL;
-  GNode *child;
+  GSList *destroy_nodes = NULL;
+  GSList *remaining_nodes = NULL;
+  GNode  *child;
 
   g_return_if_fail (node != NULL);
 
@@ -680,18 +675,19 @@ garcon_menu_merger_remove_duplicate_paths (GNode             *node,
       if (garcon_menu_node_tree_get_node_type (child) != type)
         continue;
 
-      if (G_LIKELY (g_list_find_custom (remaining_nodes, child,
-                                        (GCompareFunc) garcon_menu_node_tree_compare) == NULL))
+      if (G_LIKELY (g_slist_find_custom (remaining_nodes, child,
+                                         (GCompareFunc) garcon_menu_node_tree_compare) == NULL))
         {
-          remaining_nodes = g_list_prepend (remaining_nodes, child);
+          remaining_nodes = g_slist_prepend (remaining_nodes, child);
         }
       else
-        destroy_nodes = g_list_prepend (destroy_nodes, child);
+        {
+          destroy_nodes = g_slist_prepend (destroy_nodes, child);
+        }
     }
 
-  g_list_foreach (destroy_nodes, (GFunc) garcon_menu_node_tree_free, NULL);
-  g_list_free (destroy_nodes);
-  g_list_free (remaining_nodes);
+  _garcon_g_slist_free_full (destroy_nodes, garcon_menu_node_tree_free);
+  g_slist_free (remaining_nodes);
 }
 
 
@@ -992,8 +988,7 @@ garcon_menu_merger_clean_up_elements (GNode             *node,
         remaining_node = child;
     }
 
-  g_list_foreach (destroy_list, (GFunc) garcon_menu_node_tree_free, NULL);
-  g_list_free (destroy_list);
+  _garcon_g_list_free_full (destroy_list, garcon_menu_node_tree_free);
 
   if (type == GARCON_MENU_NODE_TYPE_LAYOUT 
       && remaining_node != NULL
@@ -1189,6 +1184,8 @@ garcon_menu_merger_resolve_moves (GNode *node)
   g_node_traverse (node, G_IN_ORDER, G_TRAVERSE_ALL, 2,
                    (GNodeTraverseFunc) remove_moves, &moves);
 
+  g_list_free (moves);
+
   for (iter = pairs; iter != NULL; iter = g_list_next (iter))
     {
       if (g_list_length (iter) < 2)
@@ -1223,8 +1220,7 @@ garcon_menu_merger_resolve_moves (GNode *node)
       g_strfreev (new_path);
     }
 
-  g_list_foreach (pairs, (GFunc) g_free, NULL);
-  g_list_free (pairs);
+  _garcon_g_list_free_full (pairs, g_free);
 }
 
 
